<?php
namespace Dugi;

use DateTime;
use DateTimeZone;
use RuntimeException;


/**
 * Usage:
 * $lm_logger = new Dugi\lm_logger('/var/log/');
 * $lm_logger->add('string');
 * $lm_logger->add(array|object);
 * $lm_logger->save();
 *
 * @author  Dagan Yaakov <admin@dugi.co.il>
 * @since   08/12/2016
 * @link    http://wp.dugi.co.il
 * @version 1.0.0
 */


class lm_logger{
	protected $options = array (
		'save_log'		 => true,
		'prefix'         => 'log_',
        'extension'      => 'log',
        'dateFormat'     => 'd-m-Y H:i:s.u',
        'logTimezone'	 => 'Asia/Jerusalem',
		'saveDelimiter'  => '>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>',
        'logFileLife' 	 => 3600,
    );
	
	private $logFilePath;
	private $log;
	
	public function __construct($logDirectory = null, array $options = array()){
        if ($logDirectory === null) 
				return false;
		
		//Force
		global $lm_options;
		$options['save_log'] = (bool)$lm_options->get('save_log');
		$options['logFileLife'] = (int)$lm_options->get('log_expiry_time');
		
		$this->options = array_merge($this->options, $options);

        $logDirectory = rtrim($logDirectory, DIRECTORY_SEPARATOR);
        if ( ! file_exists($logDirectory)) {
            mkdir($logDirectory, 0755, true);
        }

        $this->setLogFilePath($logDirectory);
        if(file_exists($this->logFilePath) && !is_writable($this->logFilePath)) {
			throw new RuntimeException('The file could not be written to. Check that appropriate permissions have been set.');
        }
		
		$this->clear();
	}
	
	function __destruct(){
		$this->save();
	}
	
	public function setLogFilePath($logDirectory) {
		$this->logFilePath = $logDirectory . DIRECTORY_SEPARATOR . $this->options['prefix'] . date('Y-m-d') . '.' . $this->options['extension'];
    }
	
	public function clear(){
		$this->log = '';
	}
	
	public function add($message){
		$datetime = $this->getTimestamp();
		 if (is_string($message)){
			 $message = trim($message);
		 }else{
			 $message = PHP_EOL . $this->_indent($this->_contextToString($message));
		 }
		
		$this->log  .= PHP_EOL . "[{$datetime}] {$message}";
		
		return $this->log;
	}
	
	public function save($saveDelimiter = null){
		if (empty($this->log))
			return false;
			
		$is_new_log = (!file_exists($this->logFilePath));
		
		$saveDelimiter = ($saveDelimiter === null) ? $this->options['saveDelimiter'] : $saveDelimiter;
		$this->log .=  PHP_EOL . $saveDelimiter;
		
		if ($this->options['save_log']){
			@file_put_contents($this->logFilePath, $this->log, FILE_APPEND);
			$this->clear();
		}
		
		
		// Delete old logs
		if ($is_new_log){
			$glob = glob(dirname($this->logFilePath). DIRECTORY_SEPARATOR .'*.'.$this->options['extension']);
			$logFileLife = (int)$this->options['logFileLife'];
			if ($logFileLife){
				$EOL = time() - $logFileLife;
				foreach ((array)$glob as $file){
					if ($EOL > filectime($file)){
						error_log("********** unlink({$file})");
					}
				}
			}
		}
		
		return true;
	}
	
	
	private function getTimestamp(){
        $originalTime = microtime(true);
        $micro = sprintf("%06d", ($originalTime - floor($originalTime)) * 1000000);
        $date = new DateTime(date('Y-m-d H:i:s.'.$micro, $originalTime));
        $date->setTimezone(new DateTimeZone($this->options['logTimezone']));
		
	   return $date->format($this->options['dateFormat']);
    }
	
	protected function _contextToString($context){
		$export = '';
        foreach ($context as $key => $value) {
            $export .= "{$key}: ";
            $export .= preg_replace(array(
                '/=>\s+([a-zA-Z])/im',
                '/array\(\s+\)/im',
                '/^  |\G  /m'
            ), array(
                '=> $1',
                'array()',
                '    '
            ), str_replace('array (', 'array(', var_export($value, true)));
            $export .= PHP_EOL;
        }
        return str_replace(array('\\\\', '\\\''), array('\\', '\''), rtrim($export));
	}

    protected function _indent($string, $indent = '    '){
        return $indent.str_replace("\n", "\n".$indent, $string);
    }
	
}
